@@ -72,7 +72,6 @@ class Backend():
         self.build = build
         self.environment = build.environment
         self.processed_targets = {}
-        self.dep_rules = {}
         self.build_to_src = os.path.relpath(self.environment.get_source_dir(),
                                             self.environment.get_build_dir())
         for t in self.build.targets:
@@ -120,6 +119,12 @@ class Backend():
             return os.path.join(self.get_target_dir(target), target.get_filename())
         raise AssertionError('BUG: Tried to link to something that\'s not a library')
 
+    def get_target_debug_filename(self, target):
+        fname = target.get_debug_filename()
+        if not fname:
+            raise AssertionError("BUG: Tried to generate debug filename when it doesn't exist")
+        return os.path.join(self.get_target_dir(target), fname)
+
     def get_target_dir(self, target):
         if self.environment.coredata.get_builtin_option('layout') == 'mirror':
             dirname = target.get_subdir()
@@ -232,27 +237,6 @@ class Backend():
             self.write_benchmark_file(datafile)
         return (test_data, benchmark_data)
 
-    def has_source_suffix(self, target, suffix):
-        for s in target.get_sources():
-            if s.endswith(suffix):
-                return True
-        return False
-
-    def has_vala(self, target):
-        return self.has_source_suffix(target, '.vala')
-
-    def has_rust(self, target):
-        return self.has_source_suffix(target, '.rs')
-
-    def has_cs(self, target):
-        return self.has_source_suffix(target, '.cs')
-
-    def has_swift(self, target):
-        return self.has_source_suffix(target, '.swift')
-
-    def has_d(self, target):
-        return self.has_source_suffix(target, '.d')
-
     def determine_linker(self, target, src):
         if isinstance(target, build.StaticLibrary):
             if self.build.static_cross_linker is not None:
@@ -475,7 +459,7 @@ class Backend():
         if self.environment.is_cross_build() and \
            self.environment.cross_info.need_exe_wrapper() and \
            isinstance(exe, build.BuildTarget) and exe.is_cross:
-            if 'exe_wrapper' not in self.environment.cross_info.config:
+            if 'exe_wrapper' not in self.environment.cross_info.config['binaries']:
                 s = 'Can not use target %s as a generator because it is cross-built\n'
                 s += 'and no exe wrapper is defined. You might want to set it to native instead.'
                 s = s % exe.name
@@ -593,6 +577,25 @@ class Backend():
                                   os.path.join(lead_dir,
                                                outdir))
                 cmd.append(i)
+        # This should not be necessary but removing it breaks
+        # building GStreamer on Windows. The underlying issue
+        # is problems with quoting backslashes on Windows
+        # which is the seventh circle of hell. The downside is
+        # that this breaks custom targets whose command lines
+        # have backslashes. If you try to fix this be sure to
+        # check that it does not break GST.
+        #
+        # The bug causes file paths such as c:\foo to get escaped
+        # into c:\\foo.
+        #
+        # Unfortunately we have not been able to come up with an
+        # isolated test case for this so unless you manage to come up
+        # with one, the only way is to test the building with Gst's
+        # setup. Note this in your MR or ping us and we will get it
+        # fixed.
+        #
+        # https://github.com/mesonbuild/meson/pull/737
+        cmd = [i.replace('\\', '/') for i in cmd]
         return (srcs, ofilenames, cmd)
 
     def run_postconf_scripts(self):
@@ -605,3 +608,24 @@ class Backend():
         for s in self.build.postconf_scripts:
             cmd = s['exe'].get_command() + s['args']
             subprocess.check_call(cmd, env=child_env)
+
+    # Subprojects of subprojects may cause the same dep args to be used
+    # multiple times. Remove duplicates here. Note that we can't dedup
+    # libraries based on name alone, because "-lfoo -lbar -lfoo" is 
+    # a completely valid (though pathological) sequence and removing the
+    # latter may fail. Usually only applies to static libs, though.
+    def dedup_arguments(self, commands):
+        includes = {}
+        final_commands = []
+        previous = '-fsuch_arguments=woof'
+        for c in commands:
+            if c.startswith(('-I', '-L', '/LIBPATH')):
+                if c in includes:
+                    continue
+                includes[c] = True
+            if previous == c:
+                continue
+            previous = c
+            final_commands.append(c)
+        return final_commands
+                                                  
