@@ -38,6 +38,12 @@ class RawFilename():
     def __init__(self, fname):
         self.fname = fname
 
+    def __str__(self):
+        return self.fname
+
+    def __repr__(self):
+        return '<RawFilename: {0}>'.format(self.fname)
+
     def split(self, c):
         return self.fname.split(c)
 
@@ -233,23 +239,19 @@ int dummy;
         if isinstance(target, build.Jar):
             self.generate_jar_target(target, outfile)
             return
-        if 'rust' in self.environment.coredata.compilers.keys() and self.has_rust(target):
+        if 'rust' in target.compilers:
             self.generate_rust_target(target, outfile)
             return
-        if 'cs' in self.environment.coredata.compilers.keys() and self.has_cs(target):
+        if 'cs' in target.compilers:
             self.generate_cs_target(target, outfile)
             return
-        if 'vala' in self.environment.coredata.compilers.keys() and self.has_vala(target):
-            vc = self.environment.coredata.compilers['vala']
-            vala_output_files = self.generate_vala_compile(vc, target, outfile)
-            gen_src_deps += vala_output_files
-        if 'swift' in self.environment.coredata.compilers.keys() and self.has_swift(target):
+        if 'swift' in target.compilers:
             self.generate_swift_target(target, outfile)
             return
+        if 'vala' in target.compilers:
+            vala_output_files = self.generate_vala_compile(target, outfile)
+            gen_src_deps += vala_output_files
         self.scan_fortran_module_outputs(target)
-        # The following deals with C/C++ compilation.
-        (gen_src, gen_other_deps) = self.process_dep_gens(outfile, target)
-        gen_src_deps += gen_src
         self.process_target_dependencies(target, outfile)
         self.generate_custom_generator_rules(target, outfile)
         outname = self.get_target_filename(target)
@@ -260,20 +262,30 @@ int dummy;
             pch_objects = self.generate_pch(target, outfile)
         else:
             pch_objects = []
-        header_deps = gen_other_deps
+        header_deps = []
         unity_src = []
         unity_deps = [] # Generated sources that must be built before compiling a Unity target.
         header_deps += self.get_generated_headers(target)
+        src_list = []
+
+        # Get a list of all generated *sources* (sources files, headers,
+        # objects, etc). Needed to determine the linker.
+        generated_output_sources = [] 
+        # Get a list of all generated headers that will be needed while building
+        # this target's sources (generated sources and pre-existing sources).
+        # This will be set as dependencies of all the target's sources. At the
+        # same time, also deal with generated sources that need to be compiled.
+        generated_source_files = []
         for gensource in target.get_generated_sources():
             if isinstance(gensource, build.CustomTarget):
                 for src in gensource.output:
                     src = os.path.join(self.get_target_dir(gensource), src)
+                    generated_output_sources.append(src)
                     if self.environment.is_source(src) and not self.environment.is_header(src):
                         if is_unity:
                             unity_deps.append(os.path.join(self.environment.get_build_dir(), RawFilename(src)))
                         else:
-                            obj_list.append(self.generate_single_compile(target, outfile, RawFilename(src), True,
-                                                                         header_deps))
+                            generated_source_files.append(RawFilename(src))
                     elif self.environment.is_object(src):
                         obj_list.append(src)
                     elif self.environment.is_library(src):
@@ -285,6 +297,7 @@ int dummy;
                         header_deps.append(RawFilename(src))
             else:
                 for src in gensource.get_outfilelist():
+                    generated_output_sources.append(src)
                     if self.environment.is_object(src):
                         obj_list.append(os.path.join(self.get_target_private_dir(target), src))
                     elif not self.environment.is_header(src):
@@ -297,9 +310,16 @@ int dummy;
                             abs_src = os.path.join(self.environment.get_build_dir(), rel_src)
                             unity_src.append(abs_src)
                         else:
-                            obj_list.append(self.generate_single_compile(target, outfile, src, True,
-                                                                         header_deps=header_deps))
-        src_list = []
+                            generated_source_files.append(src)
+        # These are the generated source files that need to be built for use by
+        # this target. We create the Ninja build file elements for this here
+        # because we need `header_deps` to be fully generated in the above loop.
+        for src in generated_source_files:
+            src_list.append(src)
+            obj_list.append(self.generate_single_compile(target, outfile, src, True,
+                                                         header_deps=header_deps))
+        # Generate compilation targets for sources belonging to this target that
+        # are generated by other rules (this is only used for Vala right now)
         for src in gen_src_deps:
             src_list.append(src)
             if is_unity:
@@ -315,6 +335,7 @@ int dummy;
                     header_deps.append(src)
                 else:
                     obj_list.append(self.generate_single_compile(target, outfile, src, True, [], header_deps))
+        # Generate compile targets for all the pre-existing sources for this target
         for src in target.get_sources():
             if src.endswith('.vala'):
                 continue
@@ -330,7 +351,7 @@ int dummy;
         if is_unity:
             for src in self.generate_unity_files(target, unity_src):
                 obj_list.append(self.generate_single_compile(target, outfile, src, True, unity_deps + header_deps))
-        linker = self.determine_linker(target, src_list)
+        linker = self.determine_linker(target, src_list + generated_output_sources)
         elem = self.generate_link(target, outfile, outname, obj_list, linker, pch_objects)
         self.generate_shlib_aliases(target, self.get_target_dir(target))
         elem.write(outfile)
@@ -424,11 +445,13 @@ int dummy;
             elif isinstance(i, (build.BuildTarget, build.CustomTarget)):
                 relfname = self.get_target_filename(i)
                 arg_strings.append(os.path.join(self.environment.get_build_dir(), relfname))
+                deps.append(relfname)
+            elif isinstance(i, mesonlib.File):
+                arg_strings.append(i.rel_to_builddir(self.build_to_src))
             else:
                 mlog.debug(str(i))
                 raise MesonException('Unreachable code in generate_run_target.')
-        elem = NinjaBuildElement(self.all_outputs, target.name, 'CUSTOM_COMMAND', deps)
-        elem.add_dep(deps)
+        elem = NinjaBuildElement(self.all_outputs, target.name, 'CUSTOM_COMMAND', [])
         cmd = runnerscript + [self.environment.get_source_dir(), self.environment.get_build_dir(), target.subdir]
         texe = target.command
         try:
@@ -446,9 +469,13 @@ int dummy;
             cmd.append(abs_exe)
         elif isinstance(texe, dependencies.ExternalProgram):
             cmd += texe.get_command()
+        elif isinstance(texe, build.CustomTarget):
+            deps.append(self.get_target_filename(texe))
+            cmd += [os.path.join(self.environment.get_build_dir(), self.get_target_filename(texe))]
         else:
             cmd.append(target.command)
         cmd += arg_strings
+        elem.add_dep(deps)
         elem.add_item('COMMAND', cmd)
         elem.add_item('description', 'Running external command %s.' % target.name)
         elem.add_item('pool', 'console')
@@ -538,6 +565,13 @@ int dummy;
                 else:
                     # XXX: Add BuildTarget-specific install dir cases here
                     outdir = self.environment.get_libdir()
+                if isinstance(t, build.SharedLibrary) or isinstance(t, build.Executable):
+                    if t.get_debug_filename():
+                        # Install the debug symbols file in the same place as
+                        # the target itself. It has no aliases, should not be
+                        # stripped, and doesn't have an install_rpath
+                        i = [self.get_target_debug_filename(t), outdir, [], False, '']
+                        d.targets.append(i)
                 i = [self.get_target_filename(t), outdir, t.get_aliaslist(),\
                     should_strip, t.install_rpath]
                 d.targets.append(i)
@@ -702,8 +736,7 @@ int dummy;
         outname_rel = os.path.join(self.get_target_dir(target), fname)
         src_list = target.get_sources()
         class_list = []
-        compiler = self.get_compiler_for_source(src_list[0], False)
-        assert(compiler.get_language() == 'java')
+        compiler = target.compilers['java']
         c = 'c'
         m = ''
         e = ''
@@ -753,8 +786,7 @@ int dummy;
         fname = target.get_filename()
         outname_rel = os.path.join(self.get_target_dir(target), fname)
         src_list = target.get_sources()
-        compiler = self.get_compiler_for_source(src_list[0], False)
-        assert(compiler.get_language() == 'cs')
+        compiler = target.compilers['cs']
         rel_srcs = [s.rel_to_builddir(self.build_to_src) for s in src_list]
         deps = []
         commands = target.extra_args.get('cs', [])
@@ -807,14 +839,14 @@ int dummy;
         outfile.write('\n')
 
     def split_vala_sources(self, sources):
-        src = []
+        other_src = []
         vapi_src = []
         for s in sources:
             if s.endswith('.vapi'):
                 vapi_src.append(s)
             else:
-                src.append(s)
-        return (src, vapi_src)
+                other_src.append(s)
+        return (other_src, vapi_src)
 
     def determine_dep_vapis(self, target):
         result = []
@@ -829,17 +861,13 @@ int dummy;
                     break
         return result
 
-    def generate_vala_compile(self, compiler, target, outfile):
+    def generate_vala_compile(self, target, outfile):
         """Vala is compiled into C. Set up all necessary build steps here."""
-        valac = self.environment.coredata.compilers['vala']
-        (src, vapi_src) = self.split_vala_sources(target.get_sources())
+        valac = target.compilers['vala']
+        (other_src, vapi_src) = self.split_vala_sources(target.get_sources())
         vapi_src = [x.rel_to_builddir(self.build_to_src) for x in vapi_src]
         extra_dep_files = []
-        vala_input_files = []
-        for s in src:
-            if s.endswith('.vala'):
-                vala_input_files.append(s.rel_to_builddir(self.build_to_src))
-        if len(src) == 0:
+        if len(other_src) == 0:
             raise InvalidArguments('Vala library has no Vala source files.')
         namebase = target.name
         base_h = namebase + '.h'
@@ -850,8 +878,8 @@ int dummy;
         generated_c_files = []
         outputs = [vapiname]
         args = []
-        args += self.build.get_global_args(compiler)
-        args += compiler.get_buildtype_args(self.environment.coredata.get_builtin_option('buildtype'))
+        args += self.build.get_global_args(valac)
+        args += valac.get_buildtype_args(self.environment.coredata.get_builtin_option('buildtype'))
         args += ['-d', self.get_target_private_dir(target)]
         args += ['-C']#, '-o', cname]
         if not isinstance(target, build.Executable):
@@ -859,9 +887,23 @@ int dummy;
             args += ['-H', hname]
             args += ['--library=' + target.name]
         args += ['--vapi=' + os.path.join('..', base_vapi)]
-        for src in vala_input_files:
-            namebase = os.path.splitext(os.path.split(src)[1])[0] + '.c'
-            full_c = os.path.join(self.get_target_private_dir(target), namebase)
+        vala_src = []
+        for s in other_src:
+            if not s.endswith('.vala'):
+                continue
+            vala_file = s.rel_to_builddir(self.build_to_src)
+            vala_src.append(vala_file)
+            # Figure out where the Vala compiler will write the compiled C file
+            dirname, basename = os.path.split(vala_file)
+            # If the Vala file is in a subdir of the build dir (in our case
+            # because it was generated/built by something else), the subdir path
+            # components will be preserved in the output path. But if the Vala
+            # file is outside the build directory, the path components will be
+            # stripped and just the basename will be used.
+            c_file = os.path.splitext(basename)[0] + '.c'
+            if s.is_built:
+                c_file = os.path.join(dirname, c_file)
+            full_c = os.path.join(self.get_target_private_dir(target), c_file)
             generated_c_files.append(full_c)
             outputs.append(full_c)
         if self.environment.coredata.get_builtin_option('werror'):
@@ -887,14 +929,14 @@ int dummy;
         args += dependency_vapis
         element = NinjaBuildElement(self.all_outputs, outputs,
                                     valac.get_language() + '_COMPILER',
-                                    vala_input_files + vapi_src)
+                                    vala_src + vapi_src)
         element.add_item('ARGS', args)
         element.add_dep(extra_dep_files)
         element.write(outfile)
         return generated_c_files
 
     def generate_rust_target(self, target, outfile):
-        rustc = self.environment.coredata.compilers['rust']
+        rustc = target.compilers['rust']
         relsrc = []
         for i in target.get_sources():
             if not rustc.can_compile(i):
@@ -985,7 +1027,7 @@ int dummy;
 
     def generate_swift_target(self, target, outfile):
         module_name = self.target_swift_modulename(target)
-        swiftc = self.environment.coredata.compilers['swift']
+        swiftc = target.compilers['swift']
         abssrc = []
         abs_headers = []
         header_imports = []
@@ -1476,6 +1518,66 @@ rule FORTRAN_DEP_HACK
             return []
         return compiler.get_no_stdinc_args()
 
+    def get_compile_debugfile_args(self, compiler, target, objfile):
+        if compiler.id != 'msvc':
+            return []
+        # The way MSVC uses PDB files is documented exactly nowhere so
+        # the following is what we have been able to decipher via
+        # reverse engineering.
+        #
+        # Each object file gets the path of its PDB file written
+        # inside it.  This can be either the final PDB (for, say,
+        # foo.exe) or an object pdb (for foo.obj). If the former, then
+        # each compilation step locks the pdb file for writing, which
+        # is a bottleneck and object files from one target can not be
+        # used in a different target. The latter seems to be the
+        # sensible one (and what Unix does) but there is a catch.  If
+        # you try to use precompiled headers MSVC will error out
+        # because both source and pch pdbs go in the same file and
+        # they must be the same.
+        #
+        # This means:
+        #
+        # - pch files must be compiled anew for every object file (negating
+        #   the entire point of having them in the first place)
+        # - when using pch, output must go to the target pdb
+        #
+        # Since both of these are broken in some way, use the one that
+        # works for each target. This unfortunately means that you
+        # can't combine pch and object extraction in a single target.
+        #
+        # PDB files also lead to filename collisions. A target foo.exe
+        # has a corresponding foo.pdb. A shared library foo.dll _also_
+        # has pdb file called foo.pdb. So will a static library
+        # foo.lib, which clobbers both foo.pdb _and_ the dll file's
+        # export library called foo.lib (by default, currently we name
+        # them libfoo.a to avoidt this issue). You can give the files
+        # unique names such as foo_exe.pdb but VC also generates a
+        # bunch of other files which take their names from the target
+        # basename (i.e. "foo") and stomp on each other.
+        #
+        # CMake solves this problem by doing two things. First of all
+        # static libraries do not generate pdb files at
+        # all. Presumably you don't need them and VC is smart enough
+        # to look up the original data when linking (speculation, not
+        # tested). The second solution is that you can only have
+        # target named "foo" as an exe, shared lib _or_ static
+        # lib. This makes filename collisions not happen. The downside
+        # is that you can't have an executable foo that uses a shared
+        # library libfoo.so, which is a common idiom on Unix.
+        #
+        # If you feel that the above is completely wrong and all of
+        # this is actually doable, please send patches.
+
+        if target.has_pch():
+            tfilename = self.get_target_filename_abs(target)
+            return compiler.get_compile_debugfile_args(tfilename)
+        else:
+            return compiler.get_compile_debugfile_args(objfile)
+
+    def get_link_debugfile_args(self, linker, target, outname):
+        return linker.get_link_debugfile_args(outname)
+
     def generate_single_compile(self, target, outfile, src, is_generated=False, header_deps=[], order_deps=[]):
         if(isinstance(src, str) and src.endswith('.h')):
             raise RuntimeError('Fug')
@@ -1562,6 +1664,8 @@ rule FORTRAN_DEP_HACK
             commands+= compiler.get_include_args(i, False)
         if self.environment.coredata.base_options.get('b_pch', False):
             commands += self.get_pch_include_args(compiler, target)
+
+        commands += self.get_compile_debugfile_args(compiler, target, rel_obj)
         crstr = ''
         if target.is_cross:
             crstr = '_CROSS'
@@ -1596,6 +1700,7 @@ rule FORTRAN_DEP_HACK
             element.add_orderdep(d)
         element.add_orderdep(pch_dep)
         element.add_orderdep(extra_orderdeps)
+        commands = self.dedup_arguments(commands)
         for i in self.get_fortran_orderdeps(target, compiler):
             element.add_orderdep(i)
         element.add_item('DEPFILE', dep_file)
@@ -1630,6 +1735,7 @@ rule FORTRAN_DEP_HACK
         just_name = os.path.split(header)[1]
         (objname, pch_args) = compiler.gen_pch_args(just_name, source, dst)
         commands += pch_args
+        commands += self.get_compile_debugfile_args(compiler, target, objname)
         dep = dst + '.' + compiler.get_depfile_suffix()
         return (commands, dep, dst, [objname])
 
@@ -1709,6 +1815,7 @@ rule FORTRAN_DEP_HACK
                                                      linker)
         commands += linker.get_buildtype_linker_args(self.environment.coredata.get_builtin_option('buildtype'))
         commands += linker.get_option_link_args(self.environment.coredata.compiler_options)
+        commands += self.get_link_debugfile_args(linker, target, outname)
         if not(isinstance(target, build.StaticLibrary)):
             commands += self.environment.coredata.external_link_args[linker.get_language()]
         if isinstance(target, build.Executable):
@@ -1756,7 +1863,7 @@ rule FORTRAN_DEP_HACK
         custom_target_libraries = self.get_custom_target_provided_libraries(target)
         commands += extra_args
         commands += custom_target_libraries
-        commands = linker.unix_link_flags_to_native(commands)
+        commands = linker.unix_link_flags_to_native(self.dedup_arguments(commands))
         dep_targets = [self.get_dependency_filename(t) for t in dependencies]
         dep_targets += [os.path.join(self.environment.source_dir,
                                      target.subdir, t) for t in target.link_depends]
@@ -1810,36 +1917,6 @@ rule FORTRAN_DEP_HACK
             gcda_elem.add_item('description', 'Deleting gcda files')
             gcda_elem.write(outfile)
 
-    def is_compilable_file(self, filename):
-        if filename.endswith('.cpp') or\
-        filename.endswith('.c') or\
-        filename.endswith('.cxx') or\
-        filename.endswith('.cc') or\
-        filename.endswith('.C'):
-            return True
-        return False
-
-    def process_dep_gens(self, outfile, target):
-        src_deps = []
-        other_deps = []
-        for rule in self.dep_rules.values():
-            srcs = target.get_original_kwargs().get(rule.src_keyword, [])
-            if isinstance(srcs, str):
-                srcs = [srcs]
-            for src in srcs:
-                plainname = os.path.split(src)[1]
-                basename = plainname.split('.')[0]
-                outname = rule.name_templ.replace('@BASENAME@', basename).replace('@PLAINNAME@', plainname)
-                outfilename = os.path.join(self.get_target_private_dir(target), outname)
-                infilename = os.path.join(self.build_to_src, target.get_source_subdir(), src)
-                elem = NinjaBuildElement(self.all_outputs, outfilename, rule.name, infilename)
-                elem.write(outfile)
-                if self.is_compilable_file(outfilename):
-                    src_deps.append(outfilename)
-                else:
-                    other_deps.append(outfilename)
-        return (src_deps, other_deps)
-
     # For things like scan-build and other helper tools we might have.
     def generate_utils(self, outfile):
         cmd = [sys.executable, self.environment.get_build_command(),
@@ -1851,8 +1928,16 @@ rule FORTRAN_DEP_HACK
         elem.write(outfile)
 
     def generate_ending(self, outfile):
-        targetlist = [self.get_target_filename(t) for t in self.build.get_targets().values()\
-                      if not isinstance(t, build.RunTarget)]
+        targetlist = []
+        for t in self.build.get_targets().values():
+            # RunTargets are meant to be invoked manually
+            if isinstance(t, build.RunTarget):
+                continue
+            # CustomTargets that aren't installed should only be built if they
+            # are used by something else or are meant to be always built
+            if isinstance(t, build.CustomTarget) and not (t.install or t.build_always):
+                continue
+            targetlist.append(self.get_target_filename(t))
 
         elem = NinjaBuildElement(self.all_outputs, 'all', 'phony', targetlist)
         elem.write(outfile)
