@@ -51,36 +51,16 @@ known_shlib_kwargs.update({'version' : True,
                            'name_suffix' : True,
                            'vs_module_defs' : True})
 
-def sources_are_suffix(sources, suffix):
-    for source in sources:
-        if source.endswith('.' + suffix):
-            return True
-    return False
-
-def compiler_is_msvc(sources, is_cross, env):
+def compilers_are_msvc(compilers):
     """
-    Since each target does not currently have the compiler information attached
-    to it, we must do this detection manually here.
-
-    This detection is purposely incomplete and will cause bugs if other code is
-    extended and this piece of code is forgotten.
+    Check if all the listed compilers are MSVC. Used by Executable,
+    StaticLibrary, and SharedLibrary for deciding when to use MSVC-specific
+    file naming.
     """
-    compiler = None
-    if sources_are_suffix(sources, 'c'):
-        try:
-            compiler = env.detect_c_compiler(is_cross)
-        except MesonException:
+    for compiler in compilers.values():
+        if compiler.get_id() != 'msvc':
             return False
-    elif sources_are_suffix(sources, 'cxx') or \
-         sources_are_suffix(sources, 'cpp') or \
-         sources_are_suffix(sources, 'cc'):
-        try:
-            compiler = env.detect_cpp_compiler(is_cross)
-        except MesonException:
-            return False
-    if compiler and compiler.get_id() == 'msvc':
-        return True
-    return False
+    return True
 
 
 class InvalidArguments(MesonException):
@@ -193,19 +173,58 @@ class ExtractedObjects():
         self.target = target
         self.srclist = srclist
 
+class EnvironmentVariables():
+    def __init__(self):
+        self.envvars = []
+
+    def get_value(self, name, values, kwargs):
+        separator = kwargs.get('separator', os.pathsep)
+
+        value = ''
+        for var in values:
+            value += separator + var
+        return separator, value.strip(separator)
+
+    def set(self, env, name, values, kwargs):
+        return self.get_value(name, values, kwargs)[1]
+
+    def append(self, env, name, values, kwargs):
+        sep, value = self.get_value(name, values, kwargs)
+        if name in env:
+            return env[name] + sep + value
+        return value
+
+    def prepend(self, env, name, values, kwargs):
+        sep, value = self.get_value(name, values, kwargs)
+        if name in env:
+            return  value + sep + env[name]
+
+        return value
+
+    def get_env(self, full_env):
+        env = {}
+        for method, name, values, kwargs in self.envvars:
+            env[name] = method(full_env, name, values, kwargs)
+        return env
+
+
 class BuildTarget():
     def __init__(self, name, subdir, subproject, is_cross, sources, objects, environment, kwargs):
         self.name = name
         self.subdir = subdir
         self.subproject = subproject # Can not be calculated from subdir as subproject dirname can be changed per project.
         self.is_cross = is_cross
+        self.environment = environment
         self.sources = []
+        self.compilers = {}
         self.objects = []
         self.external_deps = []
         self.include_dirs = []
         self.link_targets = []
         self.link_depends = []
         self.filename = 'no_name'
+        # The file with debugging symbols
+        self.debug_filename = None
         self.need_install = False
         self.pch = {}
         self.extra_args = {}
@@ -219,6 +238,7 @@ class BuildTarget():
             len(self.generated) == 0 and \
             len(self.objects) == 0:
             raise InvalidArguments('Build target %s has no sources.' % name)
+        self.process_compilers()
         self.validate_sources()
 
     def __repr__(self):
@@ -283,16 +303,52 @@ class BuildTarget():
                 msg = 'Bad source of type {!r} in target {!r}.'.format(type(s).__name__, self.name)
                 raise InvalidArguments(msg)
 
+    @staticmethod
+    def can_compile_remove_sources(compiler, sources):
+        removed = False
+        for s in sources[:]:
+            if compiler.can_compile(s):
+                sources.remove(s)
+                removed = True
+        return removed
+
+    def process_compilers(self):
+        if len(self.sources) == 0:
+            return
+        sources = list(self.sources)
+        if self.is_cross:
+            compilers = self.environment.coredata.cross_compilers
+        else:
+            compilers = self.environment.coredata.compilers
+        for lang, compiler in compilers.items():
+            if self.can_compile_remove_sources(compiler, sources):
+                self.compilers[lang] = compiler
+
     def validate_sources(self):
-        if len(self.sources) > 0:
+        if len(self.sources) == 0:
+            return
+        for lang in ('cs', 'java'):
+            if lang in self.compilers:
+                check_sources = list(self.sources)
+                compiler = self.compilers[lang]
+                if not self.can_compile_remove_sources(compiler, check_sources):
+                    m = 'No {} sources found in target {!r}'.format(lang, self.name)
+                    raise InvalidArguments(m)
+                if check_sources:
+                    m = '{0} targets can only contain {0} files:\n'.format(lang.capitalize())
+                    m += '\n'.join([repr(c) for c in check_sources])
+                    raise InvalidArguments(m)
+                # CSharp and Java targets can't contain any other file types
+                assert(len(self.compilers) == 1)
+                return
+        if 'rust' in self.compilers:
             firstname = self.sources[0]
             if isinstance(firstname, File):
                 firstname = firstname.fname
             first = os.path.split(firstname)[1]
             (base, suffix) = os.path.splitext(first)
-            if suffix == '.rs':
-                if self.name != base:
-                    raise InvalidArguments('In Rust targets, the first source file must be named projectname.rs.')
+            if suffix != '.rs' or self.name != base:
+                raise InvalidArguments('In Rust targets, the first source file must be named projectname.rs.')
 
     def get_original_kwargs(self):
         return self.kwargs
@@ -467,6 +523,15 @@ class BuildTarget():
     def get_filename(self):
         return self.filename
 
+    def get_debug_filename(self):
+        """
+        The name of the file that contains debugging symbols for this target
+
+        Returns None if there are no debugging symbols or if they are embedded
+        in the filename itself
+        """
+        return self.debug_filename
+
     def get_extra_args(self, language):
         return self.extra_args.get(language, [])
 
@@ -533,7 +598,14 @@ class BuildTarget():
                 self.external_deps.append(dep)
                 self.process_sourcelist(dep.get_sources())
             else:
-                raise InvalidArguments('Argument is not an external dependency')
+                # This is a bit of a hack. We do not want Build to know anything
+                # about the interpreter so we can't import it and use isinstance.
+                # This should be reliable enough.
+                if hasattr(dep, 'subproject'):
+                    raise InvalidArguments('''Tried to use subproject object as a dependency.
+You probably wanted to use a dependency declared in it instead. Access it
+by calling get_variable() on the subproject object.''')
+                raise InvalidArguments('Argument is not an external dependency.')
 
     def get_external_deps(self):
         return self.external_deps
@@ -715,13 +787,17 @@ class Executable(BuildTarget):
             self.prefix = ''
         if not hasattr(self, 'suffix'):
             # Executable for Windows or C#/Mono
-            if for_windows(is_cross, environment) or sources_are_suffix(self.sources, 'cs'):
+            if for_windows(is_cross, environment) or 'cs' in self.compilers:
                 self.suffix = 'exe'
             else:
                 self.suffix = ''
         self.filename = self.name
         if self.suffix:
             self.filename += '.' + self.suffix
+        # See determine_debug_filenames() in build.SharedLibrary
+        buildtype = environment.coredata.get_builtin_option('buildtype')
+        if compilers_are_msvc(self.compilers) and buildtype.startswith('debug'):
+            self.debug_filename = self.prefix + self.name + '.pdb'
 
     def type_suffix(self):
         return "@exe"
@@ -729,7 +805,7 @@ class Executable(BuildTarget):
 class StaticLibrary(BuildTarget):
     def __init__(self, name, subdir, subproject, is_cross, sources, objects, environment, kwargs):
         super().__init__(name, subdir, subproject, is_cross, sources, objects, environment, kwargs)
-        if sources_are_suffix(self.sources, 'cs'):
+        if 'cs' in self.compilers:
             raise InvalidArguments('Static libraries not supported for C#.')
         # By default a static library is named libfoo.a even on Windows because
         # MSVC does not have a consistent convention for what static libraries
@@ -742,11 +818,15 @@ class StaticLibrary(BuildTarget):
             self.prefix = 'lib'
         if not hasattr(self, 'suffix'):
             # Rust static library crates have .rlib suffix
-            if sources_are_suffix(self.sources, 'rs'):
+            if 'rust' in self.compilers:
                 self.suffix = 'rlib'
             else:
                 self.suffix = 'a'
         self.filename = self.prefix + self.name + '.' + self.suffix
+        # See determine_debug_filenames() in build.SharedLibrary
+        buildtype = environment.coredata.get_builtin_option('buildtype')
+        if compilers_are_msvc(self.compilers) and buildtype.startswith('debug'):
+            self.debug_filename = self.prefix + self.name + '.pdb'
 
     def type_suffix(self):
         return "@sta"
@@ -769,6 +849,7 @@ class SharedLibrary(BuildTarget):
             self.suffix = None
         self.basic_filename_tpl = '{0.prefix}{0.name}.{0.suffix}'
         self.determine_filenames(is_cross, environment)
+        self.determine_debug_filenames(is_cross, environment)
 
     def determine_filenames(self, is_cross, env):
         """
@@ -800,12 +881,12 @@ class SharedLibrary(BuildTarget):
         if self.prefix != None and self.suffix != None:
             pass
         # C# and Mono
-        elif sources_are_suffix(self.sources, 'cs'):
+        elif 'cs' in self.compilers:
             prefix = ''
             suffix = 'dll'
             self.filename_tpl = '{0.prefix}{0.name}.{0.suffix}'
         # Rust
-        elif sources_are_suffix(self.sources, 'rs'):
+        elif 'rust' in self.compilers:
             # Currently, we always build --crate-type=rlib
             prefix = 'lib'
             suffix = 'rlib'
@@ -817,7 +898,7 @@ class SharedLibrary(BuildTarget):
             suffix = 'dll'
             self.vs_import_filename = '{0}.lib'.format(self.name)
             self.gcc_import_filename = 'lib{0}.dll.a'.format(self.name)
-            if compiler_is_msvc(self.sources, is_cross, env):
+            if compilers_are_msvc(self.compilers):
                 # Shared library is of the form foo.dll
                 prefix = ''
                 # Import library is called foo.lib
@@ -858,6 +939,21 @@ class SharedLibrary(BuildTarget):
             self.suffix = suffix
         self.filename = self.filename_tpl.format(self)
 
+    def determine_debug_filenames(self, is_cross, env):
+        """
+        Determine the debug filename(s) using the prefix/name/etc detected in
+        determine_filenames() above.
+        """
+        buildtype = env.coredata.get_builtin_option('buildtype')
+        if compilers_are_msvc(self.compilers) and buildtype.startswith('debug'):
+            # Currently we only implement separate debug symbol files for MSVC
+            # since the toolchain does it for us. Other toolchains embed the
+            # debugging symbols in the file itself by default.
+            if self.soversion:
+                self.debug_filename = '{0.prefix}{0.name}-{0.soversion}.pdb'.format(self)
+            else:
+                self.debug_filename = '{0.prefix}{0.name}.pdb'.format(self)
+
     def process_kwargs(self, kwargs, environment):
         super().process_kwargs(kwargs, environment)
         # Shared library version
@@ -1192,7 +1288,7 @@ class ConfigurationData():
         return repr(self.values)
 
     def get(self, name):
-        return self.values[name]
+        return self.values[name]                 # (val, desc)
 
     def keys(self):
         return self.values.keys()
