@@ -47,6 +47,7 @@ add_builtin_argument('includedir')
 add_builtin_argument('datadir')
 add_builtin_argument('mandir')
 add_builtin_argument('localedir')
+add_builtin_argument('sysconfdir')
 add_builtin_argument('backend')
 add_builtin_argument('buildtype')
 add_builtin_argument('strip', action='store_true')
@@ -60,8 +61,8 @@ parser.add_argument('--cross-file', default=None,
                     help='File describing cross compilation environment.')
 parser.add_argument('-D', action='append', dest='projectoptions', default=[],
                     help='Set project options.')
-parser.add_argument('-v', '--version', action='store_true', dest='print_version', default=False,
-                    help='Print version information.')
+parser.add_argument('-v', '--version', action='version',
+                    version=coredata.version)
 parser.add_argument('directories', nargs='*')
 
 class MesonApp():
@@ -168,7 +169,8 @@ itself as required.'''
         g.generate(intr)
         g.run_postconf_scripts()
         dumpfile = os.path.join(env.get_scratch_dir(), 'build.dat')
-        pickle.dump(b, open(dumpfile, 'wb'))
+        with open(dumpfile, 'wb') as f:
+            pickle.dump(b, f)
         # Write this last since we use the existence of this file to check if
         # we generated the build file successfully, so we don't want an error
         # that pops up during generation, post-conf scripts, etc to cause us to
@@ -244,9 +246,6 @@ def run(mainfile, args):
         handshake = False
     args = mesonlib.expand_arguments(args)
     options = parser.parse_args(args)
-    if options.print_version:
-        print(coredata.version)
-        return 0
     args = options.directories
     if len(args) == 0 or len(args) > 2:
         # if there's a meson.build in the dir above, and not in the current
