@@ -20,6 +20,7 @@ import sys, struct
 SHT_STRTAB = 3
 DT_NEEDED = 1
 DT_RPATH = 15
+DT_RUNPATH = 29
 DT_STRTAB = 5
 DT_SONAME = 14
 
@@ -114,11 +115,21 @@ class Elf(DataSizes):
         self.bfile = bfile
         self.verbose = verbose
         self.bf = open(bfile, 'r+b')
-        (self.ptrsize, self.is_le) = self.detect_elf_type()
-        super().__init__(self.ptrsize, self.is_le)
-        self.parse_header()
-        self.parse_sections()
-        self.parse_dynamic()
+        try:
+            (self.ptrsize, self.is_le) = self.detect_elf_type()
+            super().__init__(self.ptrsize, self.is_le)
+            self.parse_header()
+            self.parse_sections()
+            self.parse_dynamic()
+        except:
+            self.bf.close()
+            raise
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        self.bf.close()
 
     def detect_elf_type(self):
         data = self.bf.read(6)
@@ -211,21 +222,29 @@ class Elf(DataSizes):
         self.bf.seek(strtab.val + soname.val)
         print(self.read_str())
 
-    def get_rpath_offset(self):
+    def get_entry_offset(self, entrynum):
         sec = self.find_section(b'.dynstr')
         for i in self.dynamic:
-            if i.d_tag == DT_RPATH:
+            if i.d_tag == entrynum:
                 return sec.sh_offset + i.val
         return None
 
     def print_rpath(self):
-        offset = self.get_rpath_offset()
+        offset = self.get_entry_offset(DT_RPATH)
         if offset is None:
             print("This file does not have an rpath.")
         else:
             self.bf.seek(offset)
             print(self.read_str())
 
+    def print_runpath(self):
+        offset = self.get_entry_offset(DT_RUNPATH)
+        if offset is None:
+            print("This file does not have a runpath.")
+        else:
+            self.bf.seek(offset)
+            print(self.read_str())
+
     def print_deps(self):
         sec = self.find_section(b'.dynstr')
         deps = []
@@ -257,9 +276,15 @@ class Elf(DataSizes):
                 self.bf.write(newname)
 
     def fix_rpath(self, new_rpath):
+        # The path to search for can be either rpath or runpath.
+        # Fix both of them to be sure.
+        self.fix_rpathtype_entry(new_rpath, DT_RPATH)
+        self.fix_rpathtype_entry(new_rpath, DT_RUNPATH)
+
+    def fix_rpathtype_entry(self, new_rpath, entrynum):
         if isinstance(new_rpath, str):
             new_rpath = new_rpath.encode('utf8')
-        rp_off = self.get_rpath_offset()
+        rp_off = self.get_entry_offset(entrynum)
         if rp_off is None:
             if self.verbose:
                 print('File does not have rpath. It should be a fully static executable.')
@@ -272,12 +297,12 @@ class Elf(DataSizes):
         self.bf.write(new_rpath)
         self.bf.write(b'\0'*(len(old_rpath) - len(new_rpath) + 1))
         if len(new_rpath) == 0:
-            self.remove_rpath_entry()
+            self.remove_rpath_entry(entrynum)
 
-    def remove_rpath_entry(self):
+    def remove_rpath_entry(self, entrynum):
         sec = self.find_section(b'.dynamic')
         for (i, entry) in enumerate(self.dynamic):
-            if entry.d_tag == DT_RPATH:
+            if entry.d_tag == entrynum:
                 rpentry = self.dynamic[i]
                 rpentry.d_tag = 0
                 self.dynamic = self.dynamic[:i] + self.dynamic[i+1:] + [rpentry]
@@ -293,12 +318,13 @@ def run(args):
         print('Don\'t run this unless you know what you are doing.')
         print('%s: <binary file> <prefix>' % sys.argv[0])
         exit(1)
-    e = Elf(args[0])
-    if len(args) == 1:
-        e.print_rpath()
-    else:
-        new_rpath = args[1]
-        e.fix_rpath(new_rpath)
+    with Elf(args[0]) as e:
+        if len(args) == 1:
+            e.print_rpath()
+            e.print_runpath()
+        else:
+            new_rpath = args[1]
+            e.fix_rpath(new_rpath)
     return 0
 
 if __name__ == '__main__':
